/** 
 * @file yfrobot
 * @brief YFROBOT's sensors Mind+ library.
 * @n This is a MindPlus graphics programming extension for YFROBOT's module.
 * 
 * @copyright    YFROBOT,2024
 * @copyright    MIT Lesser General Public License
 * 
 * @author [email](yfrobot@qq.com)
 * @date  2024-05-16
*/

// MicroBit 软串口不支持 115200；后续产品统一更改为9600波特率
enum BAUDLIST {
    //% block="9600"
    9600,
    //% block="115200-旧版"
    115200
}


//% color="#ea8958" iconWidth=40 iconHeight=40
namespace vc {

    //% block="VC Software Baud[BAUD] RX[rx]TX[tx] recognize speech and receive data " blockType="reporter"
    //% BAUD.shadow="dropdown" BAUD.options="BAUDLIST" BAUD.defl="BAUDLIST.9600"
    //% rx.shadow="dropdown" rx.options="SSRXD"
    //% tx.shadow="dropdown" tx.options="SSTXD"
    export function vcSpeechRecognitionSoftware(parameter: any, block: any) {
        let baud = parameter.BAUD.code;
        let rxpin = parameter.rx.code;
        let txpin = parameter.tx.code;

        if (Generator.board === 'arduino' || Generator.board === 'arduinounor3' ||
            Generator.board === 'arduinonano' || Generator.board === 'leonardo' ||
            Generator.board === 'mega2560') {  //如果是arduino 系列板，生成如下代码
            Generator.addInclude("YFVCLib_AVR_Serial", `#include "UnoSerial.h"`);
            Generator.addObject(`YFVCLib_AVR_SOFTWARE_${rxpin}_${txpin}`, `SoftwareSerial`, `yfvc_SoftwareSerial_${rxpin}_${txpin}(${rxpin}, ${txpin}); // RX, TX`);
            Generator.addObject(`YFVCLib_AVR_AVRSOFTWARE_${rxpin}_${txpin}`, `UnoSoftwareSerial`, `yfvc_AVRSoftwareSerial_${rxpin}_${txpin}(&yfvc_SoftwareSerial_${rxpin}_${txpin});`);
            Generator.addSetup(`YFVCLib_AVR_AVRSOFTWARE_begin_${rxpin}_${txpin}`, `yfvc_AVRSoftwareSerial_${rxpin}_${txpin}.begin(${baud});`);
            Generator.addCode(`yfvc_AVRSoftwareSerial_${rxpin}_${txpin}.getData()`);
        } else if (Generator.board === 'microbit') { //如果是microbit，生成如下代码
            // Generator.addInclude("include_YFVCLib_AVR", `#include "MicrobitSoftwareSerial.h"`);
            // Generator.addObject("YFVCLib_AVR_Object", `YFVCLib_AVR`, `yfvc_AVR_${rxpin}_${txpin}(SerialType::SOFTWARE, ${rxpin}, ${txpin}); // RX, TX`);
            // Generator.addSetup(`YFVCLib_AVR_initSetup`, `yfvc_AVR_${rxpin}_${txpin}.begin(${baud});`);
            // Generator.addCode(`yfvc_AVR_${rxpin}_${txpin}.getData()`);

            Generator.addInclude("YFVCLib_MB_Serial", `#include "MicroBSerial.h"`);
            // Generator.addInclude("YFVCLib_MB_SoftwareSerial", `#include "SoftwareSerial.h"`);
            Generator.addObject(`YFVCLib_MB_SOFTWARE_${rxpin}_${txpin}`, `SoftwareSerial`, `yfvc_SoftwareSerial_${rxpin}_${txpin}(${rxpin}, ${txpin}); // RX, TX`);
            Generator.addObject(`YFVCLib_MB_MBSOFTWARE_${rxpin}_${txpin}`, `YFMbSoftwareSerial`, `yfvc_MBSoftwareSerial_${rxpin}_${txpin}(&yfvc_SoftwareSerial_${rxpin}_${txpin});`);
            Generator.addSetup(`YFVCLib_MB_MBSOFTWARE_begin_${rxpin}_${txpin}`, `yfvc_MBSoftwareSerial_${rxpin}_${txpin}.begin(${baud});`);
            Generator.addCode(`yfvc_MBSoftwareSerial_${rxpin}_${txpin}.getData()`);

        } else {
            // ESP32 仅支持硬件串口
            Generator.addCode(`NO SUPPORT`);
        }

    }

    //% block="VC Hardware Serial Baud[BAUD] RX[rx]TX[tx] recognize speech and receive data " blockType="reporter"
    //% BAUD.shadow="dropdown" BAUD.options="BAUDLIST" BAUD.defl="BAUDLIST.9600"
    //% rx.shadow="dropdown" rx.options="RXD"
    //% tx.shadow="dropdown" tx.options="TXD"
    export function vcSpeechRecognitionHardware(parameter: any, block: any) {
        let baud = parameter.BAUD.code;
        let rxpin = parameter.rx.code;
        let txpin = parameter.tx.code;

        if (Generator.board === 'arduino' || Generator.board === 'arduinounor3' ||
            Generator.board === 'arduinonano' || Generator.board === 'leonardo' ||
            Generator.board === 'mega2560') {  //如果是arduino 系列板，生成如下代码
            Generator.addInclude("YFVCLib_AVR_Serial", `#include "UnoSerial.h"`);
            // Generator.addObject(`YFVCLib_AVR_SOFTWARE_${rxpin}_${txpin}`, `SoftwareSerial`, `yfvc_SoftwareSerial_${rxpin}_${txpin}(${rxpin}, ${txpin}); // RX, TX`);
            Generator.addObject(`YFVCLib_AVR_AVRHARDWARE`, `UnoHardwareSerial`, `yfvc_serial(&Serial);`);
            Generator.addSetup(`YFVCLib_AVR_AVRHARDWARE_begin`, `yfvc_serial.begin(${baud});`);
            Generator.addCode(`yfvc_serial.getData()`);
        } else if (Generator.board === 'esp32' || Generator.board === 'firebeetleesp32' || Generator.board === 'firebeetleesp32e') { //如果是掌控板或者ESP32系列，生成如下代码
            Generator.addInclude("YFVCLib_ESP_Serial", `#include "ESP32Serial.h"`);
            Generator.addObject(`YFVCLib_ESP_ESPHARDWARE_${rxpin}_${txpin}`, `YFESP32HardwareSerial`, `yfvc_ESPHardwareSerial_${rxpin}_${txpin}(&Serial2, ${rxpin}, ${txpin});// 占用Serial2，默认引脚：RX17, TX16`);
            Generator.addSetup(`YFVCLib_ESP_ESPHARDWARE_begin_${rxpin}_${txpin}`, `yfvc_ESPHardwareSerial_${rxpin}_${txpin}.begin(${baud});`);
            Generator.addCode(`yfvc_ESPHardwareSerial_${rxpin}_${txpin}.getData()`);
        } else if (Generator.board === 'microbit') { //如果是microbit,不支持
            Generator.addCode(`NO SUPPORT`);
        }


        // Generator.addSetup(`vc_initSetup`, `yfrobotSerial.begin(${baud});`);

        // Generator.addInclude(`defineserialReceiveData`, `uint8_t serialReceiveData();`)
        // Generator.addInclude(`defineserialReceiveDataFun`, ``+
        //     `uint8_t serialReceiveData() {\n`+
        //     `  uint8_t c;\n`+
        //     `  static uint8_t inBuf[3];   //数据数组\n`+         // static 必须有，在测试ESPONE 主板时，没有无法正常工作
        //     `  static uint8_t offset;\n`+
        //     `  static uint32_t checksum;\n`+
        //     `  static enum _serial_state { IDLE, HEADER_5A } c_state;\n\n`+
        //     `  while (yfrobotSerial.available()) {\n`+
        //     `    c = yfrobotSerial.read();     // 读串口缓冲区\n`+
        //     `    if (c_state == IDLE) {        // 串口状态空闲,等待 HEADER 5A 状态的到来\n`+
        //     `      if (c == 0x5A) {            // 判断是否进入 HEADER 5A\n`+
        //     `        checksum = c;             // 校验和 1  -  head\n`+
        //     `        c_state = HEADER_5A;      // 进入 HEADER_5A 状态\n`+
        //     `        offset = 0;\n`+
        //     `      } else  c_state = IDLE;     // 返回IDLE状态\n`+
        //     `    } else if (c_state == HEADER_5A && offset < 3) {\n`+
        //     `      checksum += c;              // 校验和 2  -  data\n`+
        //     `      inBuf[offset] = c;\n`+
        //     `      offset++;\n`+
        //     `    } else if (c_state == HEADER_5A && offset >= 3) {\n`+
        //     `      c_state = IDLE;             // 返回IDLE状态\n`+
        //     `      if ((checksum & 0xFF) == c) {\n`+
        //     `        return inBuf[0];          // 返回指令值\n`+
        //     `      }\n`+
        //     `    }\n`+
        //     `  }\n`+
        //     `  return 0x01;                    // 未接收数据，默认返回0x01\n`+
        //     `}`
        // );

        // Generator.addCode('serialReceiveData()');
    }

}
